@@ -1,68 +1,85 @@
 frontier <- function(
-      data, crossSectionName, timePeriodName = NULL,
-      yName, xNames = NULL, qxNames = NULL, zNames = NULL, quadHalf = TRUE,
-      modelType = ifelse( is.null( zNames ), 1, 2 ), 
-      functionType = 1,
+      yName, xNames = NULL, zNames = NULL, data,
+      modelType = ifelse( is.null( zNames ), "ECF", "EEF" ),
+      ineffDecrease = TRUE,
       logDepVar = TRUE,
-      mu = FALSE,
-      eta = FALSE,
-      iprint = 0,
-      indic = 1,
+      truncNorm = FALSE,
+      zIntercept = FALSE,
+      timeEffect = FALSE,
+      startVal = NULL,
       tol = 0.00001,
-      tol2 = 0.001,
+      maxit = 1000,
       bignum = 1.0E+16,
-      step1 = 0.00001,
-      igrid2 = 1,
-      gridno = 0.1,
-      maxit = 100,
-      startVal = NULL ) {
+      searchStep = 0.00001,
+      searchTol = 0.001,
+      searchScale = NA,
+      gridSize = 0.1,
+      gridDouble = TRUE,
+      printIter = 0 ) {
 
-   if( qxNames == "all" && !is.null( qxNames ) ) {
-      qxNames <- xNames
+   # modelType (im)
+   if( modelType %in% c( 1, "ECF" ) ) {
+      modelType <- 1
+   } else if( modelType %in% c( 2, "EEF" ) ) {
+      modelType <- 2
+   } else {
+      stop( "argument 'modelType' must be either 'ECF' or 'EEF'" )
    }
-
-   if( !modelType %in% c( 1, 2 ) ) {
-      stop( "argument 'modelType' must be either 1 or 2" )
-   }
-   if( !functionType %in% c( 1, 2 ) ) {
-      stop( "argument 'functionType' must be either 1 or 2" )
+   # ineffDecrease (ipc)
+   if( !is.logical( ineffDecrease ) || length( ineffDecrease ) != 1 ) {
+      stop( "argument 'ineffDecrease' must be a single logical value" )
    }
+   # logDepVar (il)
    if( !is.logical( logDepVar ) ) {
       stop( "argument 'logDepVar' must be logical" )
    }
-   if( !is.logical( mu ) ) {
-      stop( "argument 'mu' must be logical" )
+   # truncNorm (mu)
+   if( !is.logical( truncNorm ) ) {
+      stop( "argument 'truncNorm' must be logical" )
    }
-   if( !is.logical( eta ) ) {
-      stop( "argument 'eta' must be logical" )
+   # zIntercept (mu)
+   if( !is.logical( zIntercept ) ) {
+      stop( "argument 'zIntercept' must be logical" )
    }
-   # iprint
-   if( !is.numeric( iprint ) ) {
-      stop( "argument 'iprint' must be numeric" )
-   } else if( iprint != round( iprint ) ) {
-      stop( "argument 'iprint' must be an iteger" )
-   } else if( iprint < 0 ) {
-      stop( "argument 'iprint' must be non-negative" )
+   if( !is.logical( timeEffect ) ) {
+      stop( "argument 'timeEffect' must be logical" )
    }
-   iprint <- as.integer( iprint )
-   # indic
-   if( !is.numeric( indic ) ) {
-      stop( "argument 'indic' must be numeric" )
-   } else if( indic != round( indic ) ) {
-      stop( "argument 'indic' must be an integer" )
+   # mu: truncNorm, zIntercept
+   if( modelType == 1 ) {
+      mu <- truncNorm
+   } else {
+      mu <- zIntercept
+   }
+   # printIter (iprint)
+   if( !is.numeric( printIter ) ) {
+      stop( "argument 'printIter' must be numeric" )
+   } else if( printIter != round( printIter ) ) {
+      stop( "argument 'printIter' must be an iteger" )
+   } else if( printIter < 0 ) {
+      stop( "argument 'printIter' must be non-negative" )
+   }
+   printIter <- as.integer( printIter )
+   # searchScale (indic)
+   if( length( searchScale ) != 1 ) {
+      stop( "argument 'searchScale' must be a single logical value or NA" )
+   } else if( is.na( searchScale ) ) {
+      indic <- as.integer( 1 )
+   } else if( is.logical( searchScale ) ) {
+      indic <- as.integer( 2 - 2 * searchScale )
+   } else {
+      stop( "argument 'searchScale' must be a logical value or NA" )
    }
-   indic <- as.integer( indic )
    # tol
    if( !is.numeric( tol ) ) {
       stop( "argument 'tol' must be numeric" )
    } else if( tol < 0 ) {
       stop( "argument 'tol' must be non-negative" )
    }
-   # tol2
-   if( !is.numeric( tol2 ) ) {
-      stop( "argument 'tol2' must be numeric" )
-   } else if( tol2 < 0 ) {
-      stop( "argument 'tol2' must be non-negative" )
+   # searchTol (tol2)
+   if( !is.numeric( searchTol ) ) {
+      stop( "argument 'searchTol' must be numeric" )
+   } else if( searchTol < 0 ) {
+      stop( "argument 'searchTol' must be non-negative" )
    }
    # bignum
    if( !is.numeric( bignum ) ) {
@@ -70,52 +87,56 @@ frontier <- function(
    } else if( bignum <= 0 ) {
       stop( "argument 'bignum' must be positive" )
    }
-   # step1
-   if( !is.numeric( step1 ) ) {
-      stop( "argument 'step1' must be numeric" )
-   } else if( step1 <= 0 ) {
-      stop( "argument 'step1' must be positive" )
+   # searchStep (step1)
+   if( !is.numeric( searchStep ) ) {
+      stop( "argument 'searchStep' must be numeric" )
+   } else if( searchStep <= 0 ) {
+      stop( "argument 'searchStep' must be positive" )
    }
-   # igrid2
-   if( ! igrid2 %in% c( 1, 2 ) ) {
-      stop( "argument 'igrid2' must be either '1' or '2'" )
+   # gridDouble (igrid2)
+   if( !is.logical( gridDouble ) || length( gridDouble ) != 1 ) {
+      stop( "argument 'gridDouble' must be a single logical value" )
    }
-   # gridno
-   if( !is.numeric( gridno ) ) {
-      stop( "argument 'gridno' must be numeric" )
-   } else if( gridno <= 0 ) {
-      stop( "argument 'gridno' must be positive" )
+   # gridSize (gridno)
+   if( !is.numeric( gridSize ) ) {
+      stop( "argument 'gridSize' must be numeric" )
+   } else if( gridSize <= 0 ) {
+      stop( "argument 'gridSize' must be positive" )
    }
    # maxit
    if( !is.numeric( maxit ) ) {
       stop( "argument 'maxit' must be numeric" )
    } else if( maxit != round( maxit ) ) {
       stop( "argument 'maxit' must be an integer" )
-   } else if( maxit <= 0 ) {
-      stop( "argument 'maxit' must be positive" )
+   } else if( maxit < 0 ) {
+      stop( "argument 'maxit' must not be negative" )
    }
    maxit <- as.integer( maxit )
 
-
-   nn <- length( unique( data[[ crossSectionName ]] ) )
-   nt <- ifelse( is.null( timePeriodName ), 1,
-      length( unique( data[[ timePeriodName ]] ) ) )
+   if( "plm.dim" %in% class( data ) ) {
+      nn <- length( unique( data[[ 1 ]] ) )
+      nt <- length( unique( data[[ 2 ]] ) )
+   } else {
+      nn <- nrow( data )
+      nt <- 1
+   }
    nob <- nrow( data )
    nXvars <- length( xNames )
-   nTLvars <- length( qxNames )
-   nb <- nXvars + nTLvars * ( nTLvars + 1 ) / 2
+   nb <- nXvars
    nZvars <- length( zNames )
-   if( modelType == 2 ) {
+   if( modelType == 1 ) {
+      eta <- timeEffect
+   } else {
       eta <- nZvars
    }
 
-   dataTable <- matrix( data[[ crossSectionName ]], ncol = 1 )
-
-   # time period identifier
-   if( is.null( timePeriodName ) ) {
-      dataTable <- cbind( dataTable, rep( 1, nrow( dataTable ) ) )
+   # cross section and time period identifier
+   if( "plm.dim" %in% class( data ) ) {
+      dataTable <- matrix( as.integer( data[[ 1 ]] ), ncol = 1 )
+      dataTable <- cbind( dataTable, as.integer( data[[ 2 ]] ) )
    } else {
-      dataTable <- cbind( dataTable, data[[ timePeriodName ]] )
+      dataTable <- matrix( 1:nrow( data ), ncol = 1 )
+      dataTable <- cbind( dataTable, rep( 1, nrow( dataTable ) ) )
    }
 
    # endogenous variable
@@ -130,18 +151,6 @@ frontier <- function(
       }
    }
 
-   # exogenous variables: quadratic and interaction terms
-   if( nTLvars > 0 ) {
-      for( i in 1:nTLvars ) {
-         for( j in i:nTLvars ) {
-            dataTable <- cbind( dataTable,
-               ifelse( i == j, 1 , 2 ) * ifelse( quadHalf, 0.5, 1 ) *
-               data[[ qxNames[ i ] ]] * data[[ qxNames[ j ] ]] )
-            paramNames <- c( paramNames, paste( "beta", i, j, sep = "_" ) )
-         }
-      }
-   }
-
    # variables explaining the efficiency level
    if( nZvars > 0 ) {
       for( i in 1:nZvars ) {
@@ -161,7 +170,7 @@ frontier <- function(
    }
    returnObj <- .Fortran( "front41", 
       modelType = as.integer( modelType ),
-      functionType = as.integer( functionType ),
+      ineffDecrease = as.integer( !ineffDecrease + 1 ),
       logDepVar = as.integer( logDepVar ),
       nn = as.integer( nn ),
       nt = as.integer( nt ),
@@ -169,14 +178,14 @@ frontier <- function(
       nb = as.integer( nb ),
       mu = as.integer( mu ),
       eta = as.integer( eta ),
-      iprint = as.integer( iprint ),
+      printIter = as.integer( printIter ),
       indic = as.integer( indic ),
       tol = as.double( tol ),
-      tol2 = as.double( tol2 ),
+      searchTol = as.double( searchTol ),
       bignum = as.double( bignum ),
-      step1 = as.double( step1 ),
-      igrid2 = as.integer( igrid2 ),
-      gridno = as.double( gridno ),
+      searchStep = as.double( searchStep ),
+      gridDouble = as.integer( gridDouble ),
+      gridSize = as.double( gridSize ),
       maxit = as.integer( maxit ),
       nStartVal = as.integer( length( startVal ) ),
       startVal = as.double( startVal ),
@@ -200,9 +209,41 @@ frontier <- function(
    returnObj$nRowData <- NULL
    returnObj$nColData <- NULL
    returnObj$nParamTotal <- NULL
+   # modelType
+   if( returnObj$modelType == 1 ) {
+      returnObj$modelType <- "ECF"
+   } else {
+      returnObj$modelType <- "EEF"
+   }
+   # mu: truncNorm, zIntercept
+   if( modelType == 1 ) {
+      returnObj$truncNorm <- as.logical( returnObj$mu )
+      returnObj$mu <- NULL
+   } else {
+      returnObj$zIntercept <- as.logical( returnObj$mu )
+      returnObj$mu <- NULL
+   }
+   # eta: timeEffect, nz
+   if( modelType == 1 ) {
+      returnObj$timeEffect <- as.logical( returnObj$eta )
+      returnObj$eta <- NULL
+   } else {
+      returnObj$nz <- returnObj$eta
+      returnObj$eta <- NULL
+   }
    if( length( startVal ) == 1 ){
       returnObj$startVal <- NULL
    }
+  returnObj$ineffDecrease <- as.logical( 2 - returnObj$ineffDecrease )
+   returnObj$gridDouble <- as.logical( returnObj$gridDouble )
+   if( returnObj$indic == 2 ) {
+      returnObj$searchScale <- FALSE
+   } else if( returnObj$indic == 1 ) {
+      returnObj$searchScale <- NA
+   } else {
+      returnObj$searchScale <- TRUE
+   }
+   returnObj$indic <- NULL
    returnObj$olsParam <- returnObj$olsParam[ 1:( nb + 2 ) ]
    returnObj$olsStdEr <- returnObj$olsStdEr[ 1:( nb + 1 ) ]
    if( length( startVal ) == 1 ){
@@ -215,11 +256,23 @@ frontier <- function(
    } else {
       returnObj$gridParam <- NULL
    }
-   if( modelType == 1 && eta == FALSE ) {
+   if( modelType == 1 && timeEffect == FALSE ) {
       returnObj$effic <- returnObj$effic[ , 1, drop = FALSE ]
    }
+   # assign row names and column names to efficiency estimates
+   if( "plm.dim" %in% class( data ) ) {
+      rownames( returnObj$effic ) <- levels( data[[ 1 ]] )
+      if( modelType == 1 && timeEffect == FALSE ) {
+         colnames( returnObj$effic ) <- "efficiency"
+      } else {
+         colnames( returnObj$effic ) <- levels( data[[ 2 ]] )
+      }
+   } else {
+      rownames( returnObj$effic ) <- rownames( data )
+      colnames( returnObj$effic ) <- "efficiency"
+   }
    if( modelType == 2 ) {
-      if( mu ){
+      if( zIntercept ){
          paramNames <- c( paramNames, "delta_0" )
       }
       if( nZvars > 0 ) {
@@ -229,11 +282,11 @@ frontier <- function(
    }
    paramNames <- c( paramNames, "sigma-sq", "gamma" )
    if( modelType == 1 ) {
-      if( mu ){
+      if( truncNorm ){
          paramNames <- c( paramNames, "mu" )
       }
-      if( eta ){
-         paramNames <- c( paramNames, "eta" )
+      if( timeEffect ){
+         paramNames <- c( paramNames, "time" )
       }
    }
    names( returnObj$olsParam ) <- c( paramNames[ 1:( nb + 1 ) ],
@@ -249,6 +302,7 @@ frontier <- function(
    if( !is.null( returnObj$startVal ) ) {
       names( returnObj$startVal ) <- paramNames
    }
+   returnObj$call <- match.call()
 
    class( returnObj ) <- "frontier"
    return( returnObj )
